/*
 *  MetalMaterial.cpp
 *  Assignment5
 *
 *  Created by Zach Gildersleeve on 2/21/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "MetalMaterial.h"

MetalMaterial::MetalMaterial(const Color &newColor, int newPhongTerm) : color(newColor), phongTerm(newPhongTerm) {
	
}

void MetalMaterial::shade(Color &result, const RenderContext &context, const Ray &ray, const HitRecord &hit, int depth, const Color &attenuation) const {
	
	// P = O + tV
	Vector rayDirection = ray.getDirection();
	Point hitPosition = ray.getOrigin() + (rayDirection * hit.getT());
	
	// Call primitive to get normal - should already be normalized
	Vector normal;
	hit.getPrimitive()->normal(normal, hitPosition, hit, context, ray);
	
	//costheta = N dot V
	double cosTheta = dot(normal, rayDirection);
	
	if (cosTheta > 0)   // should this be (cosTheta < 0) or (cosTheta > 0)
		normal = -normal;
	else
		cosTheta = -cosTheta;
		
	// Schlick approximation
	// Fr = R0 + (1 - R0)(1- cosTheta)^5
	
	double sch5 = pow(1 - cosTheta, 5);

	Color Fr = color + (Color(1.0, 1.0, 1.0) - color) * sch5;
	
	Vector reflectDirection = rayDirection + normal * (2 * cosTheta);
	Ray reflectRay(hitPosition, reflectDirection);
	Color reflectedColor;
	Scene *myScene = context.scene;
	myScene->traceRay(reflectedColor, context, reflectRay, depth + 1, attenuation);
	result = Fr * reflectedColor;
	
//	Color light = myScene->getAmbient() * Kambient;	
	const Object *world = myScene->object;

	
	// color light = scene.ambient * Ka
	
	Color currentSpecColor(0, 0, 0);
	
	// loop through all the lights
	for (std::vector<Light*>::iterator iterator = myScene->lightList.begin();
		 iterator < myScene->lightList.end(); iterator++)
	{
		// get C.L (light color) and L (vector to light, normalized in light class)
		Color currentLightColor;
		double distance;
		Vector Lnormal;
		distance =(**iterator).getLight(currentLightColor, Lnormal, context, hitPosition);
		
		// cosphi = N dot Ln
		double cosPhi = dot(normal, Lnormal);
		
		if (cosPhi > 0.0)
		{
			HitRecord shadowHit(distance);
			
			Ray shadowRay(hitPosition, Lnormal);
			world->intersect(shadowHit, context, shadowRay);
			
			if (!shadowHit.getPrimitive())
			{
	//			light += currentLightColor * Kdiffuse * cosPhi;
				
				Vector H = Lnormal - rayDirection;
				H.normalize();
				
				double cosAlpha = dot(H, normal);
				if (cosAlpha > 0)
				{
					double p = pow(cosAlpha, phongTerm);
					currentSpecColor += currentLightColor * p;
				}				
			}

			
			
		}
	}
	
//	result = light * color + currentSpecColor * specColor;
	result += color * currentSpecColor;// * specColor;
	
	
}
